@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id: __init__.py 149 2008-03-28 07:05:02Z albrecht.andi $
+# $Id$
 
 """
 CrunchyFrog Package
@@ -50,6 +50,7 @@ import gobject
 import gtk
 import gtk.glade
 import gettext
+import hotshot
 import traceback
 import os
 import errno
@@ -72,11 +73,12 @@ LOG_FORMAT_APP = '%(levelname)s\t%(name)s\t%(created)f\t%(message)s'
 import logging
 logging.basicConfig(format=LOG_FORMAT_APP)
 
-if isfile(abspath(join(dirname(__file__), "../setup.py"))):
+try:
+    from dist import DATA_DIR, LOCALE_DIR
+except ImportError:
     DATA_DIR = abspath(join(dirname(__file__), "../data"))
     LOCALE_DIR = abspath(join(dirname(__file__), "../po"))
-else:
-    from dist import DATA_DIR, LOCALE_DIR
+
 PLUGIN_DIR = join(DATA_DIR, "plugins")
 USER_CONFIG_DIR = abspath(expanduser("~/.config/crunchyfrog"))
 USER_CONF = join(USER_CONFIG_DIR, "config")
@@ -92,7 +94,7 @@ USER_PLUGIN_URI = gnomevfs.get_uri_from_local_path(USER_PLUGIN_DIR)
 USER_PLUGIN_REPO = join(USER_DIR, "repo.xml")
 USER_PLUGIN_REPO_URI = gnomevfs.get_uri_from_local_path(USER_PLUGIN_REPO)
 PID_FILE = join(USER_DIR, "crunchyfrog.pid")
-    
+
 gettext.bindtextdomain("crunchyfrog", LOCALE_DIR)
 gettext.textdomain("crunchyfrog")
 gtk.glade.bindtextdomain("crunchyfrog", LOCALE_DIR)
@@ -147,11 +149,11 @@ def is_alive(pid_file):
             print "missing /proc"
             return True # no /proc, assume Listen is running
         try:
-            f = open('/proc/%d/cmdline'% pid) 
+            f = open('/proc/%d/cmdline'% pid)
         except IOError, e:
             if e.errno == errno.ENOENT:
                 return False # file/pid does not exist
-            raise 
+            raise
 
         n = f.read().lower()
         f.close()
@@ -178,11 +180,13 @@ def main():
     logger.setLevel(log_level)
     if not is_alive(PID_FILE):
         if isfile(abspath(join(dirname(__file__), "../setup.py"))):
-            props = {'app-datadir': abspath(join(dirname(__file__), '../data'))}
+            props = {'app-datadir':
+                     abspath(join(dirname(__file__), '../data'))}
         else:
             props = dict()
+        props['human-readable-name'] = release.name
         gnome.init(release.name.lower(), release.version,
-                   properties=props)       
+                   properties=props)
         app = CFApplication(options)
         app.init()
         instance = app.new_instance(args)
@@ -208,7 +212,12 @@ def main():
             sel.destroy()
         else:
             client.new_instance()
-            
-        
+
+
 if __name__ == "__main__":
-    main()
+    if "CF_PROFILE" in os.environ:
+        prof = hotshot.Profile("crunchyfrog.prof")
+        prof.runcall(main)
+        prof.close()
+    else:
+        main()
