@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id: postgres.py 187 2008-06-10 04:39:55Z albrecht.andi $
+# $Id$
 
 """PostgreSQL backend"""
 
@@ -29,6 +29,7 @@ from cf.backends.schema import *
 from cf.backends import ReferenceProvider
 from cf.datasources import DatasourceInfo
 from cf.plugins.core import DBBackendPlugin
+from cf import sqlparse
 from cf.utils import Emit
 
 import time
@@ -39,27 +40,29 @@ from gettext import gettext as _
 
 import logging
 log = logging.getLogger("PG")
-        
+
 class PgReferenceProvider(ReferenceProvider):
     name = _(u"PostgreSQL Reference")
     base_url = "http://www.postgresql.org/docs/current"
-    
+
     def get_context_help_url(self, term):
         url = "http://search.postgresql.org/search?u=%2Fdocs%2F8.3%2Finteractive%2F&q="
         url += quote_plus(term.strip())
         return url
 
 class PostgresBackend(DBBackendPlugin):
+
     id = "crunchyfrog.backend.postgres"
     name = _(u"PostgreSQL Plugin")
     description = _(u"Provides access to PostgreSQL databases")
-    
+    password_option = "password"
+
     def __init__(self, app):
         DBBackendPlugin.__init__(self, app)
         log.info("Activating PostgreSQL backend")
         self.schema = PgSchema()
         self.reference = PgReferenceProvider()
-        
+
     def _get_conn_opts_from_opts(self, opts):
         conn_opts = dict()
         if opts["database"]: conn_opts["database"] = opts["database"]
@@ -68,20 +71,20 @@ class PostgresBackend(DBBackendPlugin):
         if opts["user"]: conn_opts["user"] = opts["user"]
         if opts["password"]: conn_opts["password"] = opts["password"]
         return conn_opts
-        
+
     def shutdown(self):
         log.info("Shutting down PostgreSQL backend")
-    
+
     @classmethod
     def get_datasource_options_widgets(cls, data_widgets, initial_data=None):
         return data_widgets, ["database", "host", "port", "user", "password"]
-    
+
     @classmethod
     def get_label(cls, datasource_info):
         if not datasource_info.options.get("host", None):
             datasource_info.options["host"] = "localhost"
         return DBBackendPlugin.get_label(datasource_info)
-    
+
     def dbconnect(self, data):
         opts = self._get_conn_opts_from_opts(data)
         if data.get("ask_for_password", False):
@@ -101,7 +104,7 @@ class PostgresBackend(DBBackendPlugin):
         conn = PgConnection(self, self.app, real_conn)
         conn.threadsafety = psycopg2.threadsafety
         return conn
-    
+
     def test_connection(self, data):
         try:
             conn = self.dbconnect(data)
@@ -109,19 +112,19 @@ class PostgresBackend(DBBackendPlugin):
         except DBConnectError, err:
             return str(err)
         return None
-    
+
 class PgCursor(DbAPI2Cursor):
-    
+
     def __init__(self, *args, **kw):
         DbAPI2Cursor.__init__(self, *args, **kw)
         self._notice_tag = None
-        
+
     def _check_notices(self):
         return
         while self.connection._conn.notices:
             gobject.idle_add(self.connection.emit, "notice", self.connection._conn.notices.pop())
             #Emit(self.connection, "notice", self.connection._conn.notices.pop())
-    
+
     def execute(self, *args, **kw):
         #gtk.gdk.threads_enter()
         #self._notice_tag = gobject.timeout_add(100, self._check_notices)
@@ -131,16 +134,17 @@ class PgCursor(DbAPI2Cursor):
         #gobject.source_remove(self._notice_tag)
         #gtk.gdk.threads_leave()
         #self._check_notices()
-        
+
     def get_messages(self):
         if self._cur.statusmessage:
             return [self._cur.statusmessage]
         else:
             return []
-    
+
 class PgConnection(DbAPI2Connection):
     cursor_class = PgCursor
-    
+    sqlparse_dialect = sqlparse.DialectPSQL()
+
     def update_transaction_status(self):
         stat = self._conn.get_transaction_status()
         if stat == psycopg2.extensions.TRANSACTION_STATUS_INERROR:
@@ -154,37 +158,37 @@ class PgConnection(DbAPI2Connection):
         else:
             flag = TRANSACTION_IDLE
         self.props.transaction_state = flag
-        
+
     def get_server_info(self):
         cur = self._conn.cursor()
         cur.execute("select version()")
         ret = cur.fetchone()
         cur.close()
         return ret[0]
-    
+
 class PgLanguageCollection(Collection):
     name = _(u"Languages")
-    
+
 class PgLanguage(Node):
     name = _(u"Language")
     icon = "stock_script"
     has_children = False
-    
+
 class PgSchema(SchemaProvider):
-    
+
     def __init__(self):
         SchemaProvider.__init__(self)
-        
+
     def q(self, connection, sql):
         cur = connection.cursor()._cur
         cur.execute(sql)
         return cur.fetchall()
-        
+
     def fetch_children(self, connection, parent):
         if isinstance(parent, DatasourceInfo):
             return [SchemaCollection(),
                     PgLanguageCollection()]
-        
+
         elif isinstance(parent, SchemaCollection):
             ret = []
             sql = "select nsp.oid, nsp.nspname, dsc.description \
@@ -196,13 +200,13 @@ class PgSchema(SchemaProvider):
                 s.set_data("oid", item[0])
                 ret.append(s)
             return ret
-        
+
         elif isinstance(parent, Schema):
             return [TableCollection(schema=parent),
                     ViewCollection(schema=parent),
                     SequenceCollection(schema=parent),
                     FunctionCollection(schema=parent)]
-        
+
         elif isinstance(parent, TableCollection) \
         or isinstance(parent, ViewCollection) \
         or isinstance(parent, SequenceCollection):
@@ -229,15 +233,15 @@ class PgSchema(SchemaProvider):
                 ret.append(obj(item[1], item[2], oid=item[0],
                                has_details=has_details))
             return ret
-        
+
         elif isinstance(parent, Table):
             return [ColumnCollection(table=parent),
                     ConstraintCollection(table=parent),
                     IndexCollection(table=parent)]
-        
+
         elif isinstance(parent, View):
             return [ColumnCollection(table=parent)]
-        
+
         elif isinstance(parent, ColumnCollection):
             table = parent.get_data("table")
             ret = []
@@ -247,8 +251,8 @@ class PgSchema(SchemaProvider):
             and att.attnum >= 1" % {"tableoid" : table.get_data("oid")}
             for item in self.q(connection, sql):
                 ret.append(Column(item[1], item[2], attnum=item[0]))
-            return ret    
-        
+            return ret
+
         elif isinstance(parent, FunctionCollection):
             ret = []
             schema = parent.get_data("schema")
@@ -258,7 +262,7 @@ class PgSchema(SchemaProvider):
             for item in self.q(connection, sql):
                 ret.append(Function(item[1], item[2], oid=item[0]))
             return ret
-        
+
         elif isinstance(parent, ConstraintCollection):
             ret = []
             table = parent.get_data("table")
@@ -268,7 +272,7 @@ class PgSchema(SchemaProvider):
             for item in self.q(connection, sql):
                 ret.append(Constraint(item[1], oid=item[0]))
             return ret
-        
+
         elif isinstance(parent, IndexCollection):
             ret = []
             table = parent.get_data("table")
@@ -280,19 +284,19 @@ class PgSchema(SchemaProvider):
             for item in self.q(connection, sql):
                 ret.append(Index(item[1], item[2], oid=item[0]))
             return ret
-        
+
         elif isinstance(parent, PgLanguageCollection):
             sql = "select lan.oid, lan.lanname, dsc.description \
             from pg_language lan \
             left join pg_description dsc on dsc.objoid = lan.oid"
             return [PgLanguage(item[1], item[2], oid=item[0]) for item in self.q(connection, sql)]
-        
+
     def get_details(self, connection, obj):
         func = getattr(self, "details_%s" % obj.__class__.__name__.lower(), None)
         if func:
             return func(connection, obj)
         return None
-    
+
     def details_view(self, connection, view):
         ret = dict()
         sql = "select pg_get_viewdef(%(oid)s, true)" %{"oid" : view.get_data("oid")}
@@ -305,4 +309,4 @@ try:
     import psycopg2
     import psycopg2.extensions
 except ImportError:
-    PostgresBackend.INIT_ERROR = _(u"Python module psycopg2 required.")
\ No newline at end of file
+    PostgresBackend.INIT_ERROR = _(u"Python module psycopg2 required.")
