@@ -17,15 +17,20 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """An experimental docking implementation.
-
 """
 
-import gtk
-import gobject
-import pango
-from lxml import etree
 from cStringIO import StringIO
+import math
+try:
+    from xml.etree import ElementTree as etree
+except ImportError:
+    from lxml import etree
+
 import cairo
+import gobject
+import gtk
+import pango
+
 
 DOCK_ITEM_STATUS_DOCKED = 'docked'
 DOCK_ITEM_STATUS_HIDDEN = 'hidden'
@@ -42,7 +47,7 @@ DOCK_ITEM_BEH_LOCKED = 1 << 5
 
 class Dock(gtk.VBox):
     """The docking widget."""
-    
+
     def __init__(self):
         self.__gobject_init__()
         self.items = list()
@@ -58,25 +63,41 @@ class Dock(gtk.VBox):
         hbox.pack_start(self.bar_right, False, False, 0)
         self.pack_start(hbox, True, True, 0)
         self.pack_start(self.bar_bottom, False, False, 0)
-        
-    def add_item(self, item):
-        self.main.add_item(item)
-        
-    def find_group_by_position(self, dock_item, x, y):
+
+    def _find_group_by_position(self, dock_item, x, y):
+        """Finds a dock item by x,y coordinates.
+
+        Args:
+          dock_item: A DockItem instance.
+          x,y: Coordinates.
+
+        Returns:
+          A 2-tuple.
+        """
         gx, gy = self.main.window.get_origin()
         if x >= gx and x <= gx+self.main.allocation.width \
         and y >= gy and y <= gy+self.main.allocation.width:
-            return self.main.find_group_by_position(dock_item, x, y)
+            return self.main._find_group_by_position(dock_item, x, y)
         else:
             return None, None
-            
+
+    def add_item(self, item):
+        """Adds an DockItem to the Dock.
+
+        Args:
+          item: A DockItem instance.
+        """
+        assert isinstance(item, DockItem)
+        self.main.add_item(item)
+
     def flatten_groups(self):
+        # TODO(andi): Make this a private method.
         return self.main.flatten_groups()
-    
+
     def update_views(self):
         for group in self.flatten_groups():
             group.update_views()
-                
+
     def save_layout(self, fp):
         dom = etree.Element("dock-layout")
         self.main.save_layout(dom)
@@ -96,16 +117,16 @@ class Dock(gtk.VBox):
                 fl.append(x)
                 dom.append(fl)
         fp.write(etree.tounicode(dom, pretty_print=True))
-        
+
     def dump(self):
         fp = StringIO()
         self.save_layout(fp)
         print fp.getvalue()
-        
-        
+
+
 class DockBar(gtk.HBox):
     """Container for *hidden* DockItems."""
-    
+
     def __init__(self, dock, pos):
         gtk.HBox.__init__(self)
         self.dock = dock
@@ -116,7 +137,7 @@ class DockBar(gtk.HBox):
         else:
             self.bar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
         self.pack_start(self.bar, True, True)
-            
+
     def add_item(self, item):
         btn = gtk.ToolButton()
         btn.set_data("item", item)
@@ -124,7 +145,8 @@ class DockBar(gtk.HBox):
         btn.set_label_widget(lbl)
         if item.icon:
             it = gtk.icon_theme_get_default()
-            pb = it.load_icon(item.icon, gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG)
+            pb = it.load_icon(item.icon, gtk.ICON_SIZE_MENU,
+                              gtk.ICON_LOOKUP_FORCE_SVG)
             i = gtk.Image()
             i.set_from_pixbuf(pb)
             btn.set_icon_widget(i)
@@ -133,27 +155,29 @@ class DockBar(gtk.HBox):
         self.bar.insert(btn, -1)
         self.show_all()
         btn.connect("clicked", self.on_button_clicked)
-        
+
     def on_button_clicked(self, btn):
+        # TODO(andi): Make this method private.
         btn.get_data("item").show()
         btn.get_parent().remove(btn)
         btn.destroy()
         if not self.bar.get_n_items():
             self.hide()
-            
+
+
 class DockNotebook(gtk.Notebook):
     """The main view within a DockGroup"""
-    
+
     def __init__(self):
         self.__gobject_init__()
         self.set_tab_pos(gtk.POS_BOTTOM)
         self.set_scrollable(True)
-        self.set_homogeneous_tabs(False)
         self.popup_disable()
         self.connect("page-removed", self.on_page_count_changed)
         self.connect("page-added", self.on_page_count_changed)
-        
+
     def on_page_count_changed(self, *args):
+        # TODO(andi): Make this method private.
         self.set_show_tabs(self.get_n_pages() > 1)
         self.set_show_border(self.get_n_pages() > 1)
 
@@ -161,7 +185,7 @@ class DockNotebook(gtk.Notebook):
 class DockGroup(gtk.EventBox):
     """A DockGroup holds a centered main view and up to four
     DockGroups (top, bottom, right, left)"""
-    
+
     def __init__(self, dock, name):
         self.__gobject_init__()
         self.dock = dock
@@ -182,7 +206,7 @@ class DockGroup(gtk.EventBox):
         self.group_left = None
         self.group_right = None
         self.connect("remove", self.on_remove)
-        
+
     def _get_subgroups(self):
         r = list()
         for item in ["top", "bottom", "left", "right"]:
@@ -191,8 +215,9 @@ class DockGroup(gtk.EventBox):
                 r.append(group)
         return r
     subgroups = property(fget=_get_subgroups)
-        
+
     def on_remove(self, *args):
+        # TODO(andi): Make this method private.
         if not self.group_bottom and not self.group_top \
         and not self.group_left and not self.group_right \
         and not self.notebook.get_n_pages() \
@@ -200,11 +225,13 @@ class DockGroup(gtk.EventBox):
             self.get_parent().remove(self)
             self.destroy()
             return
-            
+
     def on_group_destroyed(self, widget, group):
+        # TODO(andi): Make this method private.
         gobject.idle_add(setattr, self, group, None)
-        
+
     def flatten_groups(self):
+        # TODO(andi): Make this method private.
         groups=[]
         if self.group_bottom:
             groups += self.group_bottom.flatten_groups()
@@ -216,13 +243,13 @@ class DockGroup(gtk.EventBox):
             groups += self.group_right.flatten_groups()
         groups.append(self)
         return groups
-    
-    def find_group_by_position(self, dock_item, x, y):
+
+    def _find_group_by_position(self, dock_item, x, y):
         for group in self.subgroups:
             gx, gy = group.window.get_origin()
             if x >= gx and x <= gx+group.allocation.width \
             and y >= gy and y <= gy+group.allocation.height:
-                return group.find_group_by_position(dock_item, x, y)
+                return group._find_group_by_position(dock_item, x, y)
         # no subgroups, if it's in notebook there's nothing to do
         if dock_item in self.notebook.get_children():
             return self, None
@@ -242,7 +269,7 @@ class DockGroup(gtk.EventBox):
         else:
             pos = None
         return self, pos
-        
+
     def add_item(self, item, pos=-1):
         if pos == -1:
             pos = item.initial_position
@@ -256,7 +283,8 @@ class DockGroup(gtk.EventBox):
             lbl.set_max_width_chars(30)
             self.notebook.append_page(item, lbl)
             self.notebook.set_tab_reorderable(item, True)
-            gobject.idle_add(self.notebook.set_current_page, self.notebook.get_n_pages()-1)
+            gobject.idle_add(self.notebook.set_current_page,
+                             self.notebook.get_n_pages()-1)
         elif pos == gtk.POS_TOP:
             if self.group_top:
                 self.group_top.add_item(item, pos=None)
@@ -264,7 +292,8 @@ class DockGroup(gtk.EventBox):
                 self.group_top = DockGroup(self.dock, "top")
                 self.group_top.add_item(item, pos=None)
                 self.vpaned1.pack1(self.group_top)
-                self.group_top.connect("destroy", self.on_group_destroyed, "group_top")
+                self.group_top.connect("destroy", self.on_group_destroyed,
+                                       "group_top")
         elif pos == gtk.POS_BOTTOM:
             if self.group_bottom:
                 self.group_bottom.add_item(item, pos=None)
@@ -272,7 +301,8 @@ class DockGroup(gtk.EventBox):
                 self.group_bottom = DockGroup(self.dock, "bottom")
                 self.group_bottom.add_item(item, pos=None)
                 self.vpaned2.pack2(self.group_bottom, False, False)
-                self.group_bottom.connect("destroy", self.on_group_destroyed, "group_bottom")
+                self.group_bottom.connect("destroy", self.on_group_destroyed,
+                                          "group_bottom")
         elif pos == gtk.POS_LEFT:
             if self.group_left:
                 self.group_left.add_item(item, pos=None)
@@ -280,7 +310,8 @@ class DockGroup(gtk.EventBox):
                 self.group_left = DockGroup(self.dock, "left")
                 self.group_left.add_item(item, pos=None)
                 self.hpaned1.pack1(self.group_left)
-                self.group_left.connect("destroy", self.on_group_destroyed, "group_left")
+                self.group_left.connect("destroy", self.on_group_destroyed,
+                                        "group_left")
         elif pos == gtk.POS_RIGHT:
             if self.group_right:
                 self.group_right.add_item(item, pos=None)
@@ -288,10 +319,11 @@ class DockGroup(gtk.EventBox):
                 self.group_right = DockGroup(self.dock, "right")
                 self.group_right.add_item(item, pos=None)
                 self.hpaned2.pack2(self.group_right)
-                self.group_right.connect("destroy", self.on_group_destroyed, "group_right")
+                self.group_right.connect("destroy", self.on_group_destroyed,
+                                         "group_right")
         self.show_all()
         self.dock.update_views()
-        
+
     def relocate_item(self, item, pos):
         parent = item.get_parent()
         if isinstance(parent, gtk.Notebook):
@@ -303,7 +335,7 @@ class DockGroup(gtk.EventBox):
             item.get_parent().remove(item)
             item.destroy()
         self.add_item(item, pos)
-        
+
     def has_items(self):
         if self.notebook.get_n_pages() != 0:
             return True
@@ -352,9 +384,9 @@ class DockGroup(gtk.EventBox):
                 items.append(item)
             for item in items:
                 self.add_item(item, pos=None)
-            
-    
+
     def show_dock_target(self, win, pos):
+        # TODO(andi): Make this method private.
         x = y = width = height = None
         dx, dy = self.window.get_origin()
         if pos == gtk.POS_TOP:
@@ -387,8 +419,9 @@ class DockGroup(gtk.EventBox):
             win.show()
         else:
             win.hide()
-            
+
     def save_layout(self, node, pos=None):
+        # TODO(andi): Make this method private.
         grp = etree.Element("group")
         if pos:
             grp.set("pos", pos)
@@ -408,10 +441,11 @@ class DockGroup(gtk.EventBox):
             self.group_left.save_layout(grp, "left")
         if self.group_right:
             self.group_right.save_layout(grp, "right")
-        
+
+
 class DockItem(gtk.VBox):
     """A DockItem"""
-    
+
     def __init__(self, dock, id, widget, label, icon=None, pos=None,
                  behavior=DOCK_ITEM_BEH_NORMAL):
         self.__gobject_init__()
@@ -421,11 +455,11 @@ class DockItem(gtk.VBox):
         self.label = label
         self.icon = icon
         self.behavior = behavior
-        self.shadow_window = None
+        self.shadow_window = None # TODO(andi): Make this private.
         self.status = None
-        self.initial_position = pos
+        self.initial_position = pos # TODO(andi): Make this private.
         self._setup_widget()
-        
+
     def _setup_widget(self):
         self.header = gtk.EventBox()
         style = self.get_style()
@@ -436,7 +470,8 @@ class DockItem(gtk.VBox):
         if self.icon:
             #i = gtk.image_new_from_stock(self.icon, gtk.ICON_SIZE_MENU)
             it = gtk.icon_theme_get_default()
-            pb = it.load_icon(self.icon, gtk.ICON_SIZE_MENU, gtk.ICON_LOOKUP_FORCE_SVG)
+            pb = it.load_icon(self.icon, gtk.ICON_SIZE_MENU,
+                              gtk.ICON_LOOKUP_FORCE_SVG)
             i = gtk.Image()
             i.set_from_pixbuf(pb)
             hbox.pack_start(i, False, False)
@@ -450,7 +485,8 @@ class DockItem(gtk.VBox):
                 btn.set_property("can-focus", False)
                 btn.set_size_request(17, 17)
                 btn.set_border_width(0)
-                btn.set_image(gtk.image_new_from_stock("gtk-leave-fullscreen", gtk.ICON_SIZE_MENU))
+                btn.set_image(gtk.image_new_from_stock("gtk-leave-fullscreen",
+                                                       gtk.ICON_SIZE_MENU))
                 btn.connect("clicked", self.on_send_to_bar)
                 hbox.pack_start(btn, False, False)
             if self.behavior & DOCK_ITEM_BEH_CANT_CLOSE == 0:
@@ -458,16 +494,22 @@ class DockItem(gtk.VBox):
                 btn.set_relief(gtk.RELIEF_NONE)
                 btn.set_property("can-focus", False)
                 btn.set_size_request(17, 17)
-                btn.set_image(gtk.image_new_from_stock("gtk-close", gtk.ICON_SIZE_MENU))
+                btn.set_image(gtk.image_new_from_stock("gtk-close",
+                                                       gtk.ICON_SIZE_MENU))
                 btn.connect("clicked", self.on_close)
                 hbox.pack_start(btn, False, False)
         self.header.add(hbox)
         if self.behavior & DOCK_ITEM_BEH_LOCKED == 0:
-            self.header.connect("enter-notify-event", self.on_header_enter)
-            self.header.connect("leave-notify-event", self.on_header_leave)
-            self.header.connect("button-press-event", self.on_header_button_press)
-            self.header.connect("motion-notify-event", self.on_header_motion)
-            self.header.connect("button-release-event", self.on_header_button_release)
+            self.header.connect("enter-notify-event",
+                                self.on_header_enter)
+            self.header.connect("leave-notify-event",
+                                self.on_header_leave)
+            self.header.connect("button-press-event",
+                                self.on_header_button_press)
+            self.header.connect("motion-notify-event",
+                                self.on_header_motion)
+            self.header.connect("button-release-event",
+                                self.on_header_button_release)
         self.header.connect("realize", self.on_header_realize)
         self.pack_start(self.header, False, False)
         if self.widget.get_parent():
@@ -478,33 +520,40 @@ class DockItem(gtk.VBox):
         self.dock.items.append(self)
         self.widget.connect("destroy", self.on_widget_destroy)
         self.connect("destroy", self.on_destroy)
-            
+
     def on_close(self, *args):
+        # TODO(andi): Make this method private.
         self.destroy()
-        
+
     def on_destroy(self, *args):
+        # TODO(andi): Make this method private.
         self.dock.items.remove(self)
-        
+
     def on_send_to_bar(self, *args):
+        # TODO(andi): Make this method private.
         self.send_to_bar()
-        
+
     def on_header_enter(self, *args):
+        # TODO(andi): Make this method private.
         style = self.get_style()
         self.header.modify_bg(gtk.STATE_NORMAL, style.bg[gtk.STATE_PRELIGHT])
-        
+
     def on_header_leave(self, *args):
+        # TODO(andi): Make this method private.
         style = self.get_style()
         self.header.modify_bg(gtk.STATE_NORMAL, style.bg[gtk.STATE_ACTIVE])
-        
+
     def on_header_button_press(self, widget, event):
+        # TODO(andi): Make this method private.
         if event.button == 1:
             self.shadow_window = PlaceholderWindow(self.dock, self)
             self.header.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))
-            
+
     def on_header_button_release(self, widget, event):
+        # TODO(andi): Make this method private.
         if event.button == 1:
             x, y = event.get_root_coords()
-            group, pos = self.dock.find_group_by_position(self, x, y)
+            group, pos = self.dock._find_group_by_position(self, x, y)
             if group:
                 group.relocate_item(self, pos)
             else:
@@ -512,21 +561,26 @@ class DockItem(gtk.VBox):
             self.shadow_window.destroy()
             self.shadow_window = None
             gobject.idle_add(self.dock.update_views)
-            
+
     def on_header_motion(self, widget, event):
+        # TODO(andi): Make this method private.
         x, y = event.get_root_coords()
-        group, pos = self.dock.find_group_by_position(self, x, y)
+        group, pos = self.dock._find_group_by_position(self, x, y)
         if group:
             gobject.idle_add(group.show_dock_target, self.shadow_window, pos)
         elif self.behavior & DOCK_ITEM_BEH_NEVER_FLOATING == 0:
-            gobject.idle_add(self.shadow_window.relocate, x, y, self.allocation.width, self.allocation.height, True)
-            
+            gobject.idle_add(self.shadow_window.relocate, x, y,
+                             self.allocation.width, self.allocation.height,
+                             True)
+
     def on_header_realize(self, header, *args):
+        # TODO(andi): Make this method private.
         header.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
-        
+
     def on_widget_destroy(self, *args):
+        # TODO(andi): Make this method private.
         gobject.idle_add(self.destroy)
-        
+
     def send_to_bar(self):
         self.hide()
         parent = self.get_parent()
@@ -545,8 +599,9 @@ class DockItem(gtk.VBox):
         else:
             bar = self.dock.bar_bottom
         bar.add_item(self)
-        
+
     def set_floating(self, x, y):
+        # TODO(andi): Make this method private?
         width = self.allocation.width
         height = self.allocation.height
         w = FloatingWindow(self)
@@ -555,10 +610,14 @@ class DockItem(gtk.VBox):
         w.show_all()
         self.status = DOCK_ITEM_STATUS_FLOATING
 
+
 # Parts of the PlaceholderWindow are ported from MonoDevelop's
-# docking implementation.       
+# docking implementation.
+
+
 class PlaceholderWindow(gtk.Window):
     """A *dummy* window which is displayed when dragging around DockItems"""
+
     redgc = None
     anim = 0
     rx = 0
@@ -566,9 +625,8 @@ class PlaceholderWindow(gtk.Window):
     rw = 0
     rh = 0
     allow_docking = False
-    
     with_preview = False
-    
+
     def __init__(self, dock, dockitem):
         gtk.Window.__init__(self, gtk.WINDOW_POPUP)
         self.use_alpha = False
@@ -580,7 +638,8 @@ class PlaceholderWindow(gtk.Window):
         if self.with_preview:
             pb = gtk.gdk.pixbuf_get_from_drawable(None,
                         dockitem.window, dockitem.window.get_colormap(),
-                        0, 0, 0, 0, dockitem.allocation.width, dockitem.allocation.height)
+                        0, 0, 0, 0, dockitem.allocation.width,
+                        dockitem.allocation.height)
             f1 = 150.0/dockitem.allocation.width
             f2 = 250.0/dockitem.allocation.height
             if f1 < f2:
@@ -602,7 +661,7 @@ class PlaceholderWindow(gtk.Window):
             self.connect("expose-event", self.expose)
         else:
             self.connect("expose-event", self.expose_simple)
-        
+
     def _set_colormap(self):
         screen = self.get_screen()
         colormap = screen.get_rgba_colormap()
@@ -618,12 +677,12 @@ class PlaceholderWindow(gtk.Window):
         if x != self.rx or y != self.ry or w != self.rw or h != self.rh:
             self.move(x, y)
             self.resize(w, h)
-            
+
             self.rx = x
             self.ry = y
             self.rw = w
             self.rh = h
-            
+
             if self.anim != 0:
                 gobject.source_remove(self.anim)
                 self.anim = 0
@@ -645,32 +704,32 @@ class PlaceholderWindow(gtk.Window):
             self.resize(cw, ch)
             return True
         self.anim = 0
-        return False 
-    
+        return False
+
     def draw(self, widget, cr):
         rect = widget.get_allocation()
         draw_rb(cr, rect.x, rect.y, rect.width, rect.height, 5)
         cr.set_operator(cairo.OPERATOR_OVER)
         cr.set_source_rgba(0, 0, 0, .2)
         cr.fill_preserve()
-            
+
     def expose(self, widget, event):
         cr = widget.window.cairo_create()
         cr.rectangle(event.area.x, event.area.y,
                      event.area.width, event.area.height)
         cr.clip()
         cr.set_source_rgba(0, 0, 0, .2)
-        cr.set_operator( cairo.OPERATOR_SOURCE )
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
         self.draw(widget, cr)
         return False
-    
+
     def expose_simple(self, widget, event):
         cr = widget.window.cairo_create()
         cr.rectangle(event.area.x, event.area.y,
                      event.area.width, event.area.height)
         cr.clip()
-        cr.set_operator( cairo.OPERATOR_SOURCE )
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.set_source_rgb(0, 0, 0)
         width = self.allocation.width
         height = self.allocation.height
@@ -681,9 +740,11 @@ class PlaceholderWindow(gtk.Window):
         cr.rel_line_to(-width, 0)
         cr.stroke()
 
-# draw_cb is ported from GNOME Launch Box 
+
+# draw_cb is ported from GNOME Launch Box
+
+
 def draw_rb(cr, x, y, width, height, radius):
-    import math
     width -= 2 * radius
     height -= 2 * radius
 
@@ -692,28 +753,29 @@ def draw_rb(cr, x, y, width, height, radius):
     cr.rel_line_to(width, 0.0)
 
     cx, cy = cr.get_current_point()
-    cr.arc (cx, cy + radius, radius, 3.0 * math.pi/2, 0)
+    cr.arc(cx, cy + radius, radius, 3.0 * math.pi/2, 0)
 
-    cr.rel_line_to (0.0, height)
-
-    cx, cy = cr.get_current_point ()
-    cr.arc (cx - radius, cy, radius, 0, math.pi/2)
-
-    cr.rel_line_to (- width, 0.0)
+    cr.rel_line_to(0.0, height)
 
     cx, cy = cr.get_current_point()
-    cr.arc (cx, cy - radius, radius, math.pi/2, math.pi)
+    cr.arc(cx - radius, cy, radius, 0, math.pi/2)
 
-    cr.rel_line_to (0.0, -height)
+    cr.rel_line_to(- width, 0.0)
 
-    cx, cy = cr.get_current_point ()
-    cr.arc (cx + radius, cy, radius, math.pi, 3.0 * math.pi/2)
+    cx, cy = cr.get_current_point()
+    cr.arc(cx, cy - radius, radius, math.pi/2, math.pi)
+
+    cr.rel_line_to(0.0, -height)
+
+    cx, cy = cr.get_current_point()
+    cr.arc(cx + radius, cy, radius, math.pi, 3.0 * math.pi/2)
 
     cr.close_path()
-    
+
+
 class FloatingWindow(gtk.Window):
     """A window for floating DockItems"""
-    
+
     def __init__(self, item):
         gtk.Window.__init__(self)
         self.item = item
@@ -721,7 +783,8 @@ class FloatingWindow(gtk.Window):
         self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
         self.item.reparent(self)
         self.connect("remove", self.on_remove)
-        
+
     def on_remove(self, *args):
+        # TODO(andi): Make this method private.
         if not self.get_child():
-            self.destroy()
\ No newline at end of file
+            self.destroy()
