@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# $Id: prefs.py 187 2008-06-10 04:39:55Z albrecht.andi $
+# $Id$
 
 """Preferences"""
 
@@ -36,7 +36,10 @@ log = logging.getLogger("PREFS")
 
 from inspect import isclass
 
-from lxml import etree
+try:
+    from xml.etree import ElementTree as etree
+except ImportError:
+    from lxml import etree
 from kiwi.ui import dialogs
 
 import cf
@@ -45,12 +48,19 @@ from cf.ui.widgets import ProgressDialog
 from cf.plugins.core import GenericPlugin
 
 class PreferencesDialog(GladeWidget):
-    
-    def __init__(self, app):
+
+    def __init__(self, app, mode="editor"):
         GladeWidget.__init__(self, app, "crunchyfrog", "preferences_dialog")
         self.sync_repo_file(silent=True)
         self.refresh()
-        
+        if mode == "editor":
+            curr_page = 1
+        elif mode == "plugins":
+            curr_page = 2
+        else:
+            curr_page = 0
+        self.xml.get_widget("notebook1").set_current_page(curr_page)
+
     def _setup_widget(self):
         self._setup_plugins()
         self._setup_editor()
@@ -58,7 +68,7 @@ class PreferencesDialog(GladeWidget):
         btn = self.xml.get_widget("btn_help")
         box = self.xml.get_widget("dialog-action_area1")
         box.set_child_secondary(btn, True)
-        
+
     def _setup_editor(self):
         model = gtk.ListStore(int, str, gtk.gdk.Pixbuf)
         iconview = self.xml.get_widget("editor_iconview")
@@ -73,38 +83,30 @@ class PreferencesDialog(GladeWidget):
         model.append([2, _(u"Editor"), get_pb("gtk-edit")])
         model.append([3, _(u"Fonts & Colors"), get_pb("gtk-font")])
         iconview.connect("selection-changed", self.on_editor_selection_changed)
-        from cf.ui.widgets.sqlview import USE_GTKSOURCEVIEW2
-        if not USE_GTKSOURCEVIEW2:
-            self.xml.get_widget("editor_schemes_box").set_sensitive(False)
-            self.xml.get_widget("editor_schemes_box").hide()
-            # FIXME: Is there really no way to set right margin position on SourceView?
-            self.xml.get_widget("editor_right_margin_position_box").set_sensitive(False)
-            self.xml.get_widget("editor_right_margin_position_box").hide()
-        else:
-            schemes = self.xml.get_widget("editor_schemes")
-            model = gtk.ListStore(str, str)
-            model.set_sort_column_id(1, gtk.SORT_ASCENDING)
-            schemes.set_model(model)
-            col = gtk.TreeViewColumn("", gtk.CellRendererText(), markup=1)
-            schemes.append_column(col)
-            import gtksourceview2
-            sm = gtksourceview2.style_scheme_manager_get_default()
-            selected = None
-            for id in sm.get_scheme_ids():
-                scheme = sm.get_scheme(id)
-                lbl = "<b>%s</b>" % scheme.get_name()
-                if scheme.get_description():
-                    lbl += "\n"+scheme.get_description()
-                iter = model.append(None)
-                model.set(iter, 0, id, 1, lbl)
-                if id == self.app.config.get("editor.scheme"):
-                    selected = iter
-            sel = schemes.get_selection()
-            sel.select_iter(selected)
-            sel.connect("changed", self.on_editor_option_changed)
-            sel.set_data("config_option", "editor.scheme")
-                
-        
+        schemes = self.xml.get_widget("editor_schemes")
+        model = gtk.ListStore(str, str)
+        model.set_sort_column_id(1, gtk.SORT_ASCENDING)
+        schemes.set_model(model)
+        col = gtk.TreeViewColumn("", gtk.CellRendererText(), markup=1)
+        schemes.append_column(col)
+        import gtksourceview2
+        sm = gtksourceview2.style_scheme_manager_get_default()
+        selected = None
+        for id in sm.get_scheme_ids():
+            scheme = sm.get_scheme(id)
+            lbl = "<b>%s</b>" % scheme.get_name()
+            if scheme.get_description():
+                lbl += "\n"+scheme.get_description()
+            iter = model.append(None)
+            model.set(iter, 0, id, 1, lbl)
+            if id == self.app.config.get("editor.scheme"):
+                selected = iter
+        sel = schemes.get_selection()
+        sel.select_iter(selected)
+        sel.connect("changed", self.on_editor_option_changed)
+        sel.set_data("config_option", "editor.scheme")
+
+
     def _setup_plugins(self):
         """Set up the plugins view"""
         self.plugin_model = gtk.TreeStore(gobject.TYPE_PYOBJECT, # 0 Plugin class
@@ -132,14 +134,14 @@ class PreferencesDialog(GladeWidget):
         self.plugin_list.append_column(col)
         sel = self.plugin_list.get_selection()
         sel.connect("changed", self.on_plugin_selection_changed)
-        
+
     def _setup_connections(self):
         self.app.plugins.connect("plugin-added", self.on_plugin_added)
         self.app.plugins.connect("plugin-removed", self.on_plugin_removed)
-        
+
     def on_editor_open_in_window_toggled(self, toggle):
         self.app.config.set("editor.open_in_window", toggle.get_active())
-        
+
     def on_editor_option_changed(self, widget, *args):
         option = widget.get_data("config_option")
         conf = self.app.config
@@ -160,14 +162,14 @@ class PreferencesDialog(GladeWidget):
             self.xml.get_widget("editor_font_box").set_sensitive(not widget.get_active())
         if option == "plugins.repo_enabled":
             gobject.idle_add(self.refresh_plugins)
-        
+
     def on_editor_selection_changed(self, iconview):
         model = iconview.get_model()
         for path in iconview.get_selected_items():
             iter = model.get_iter(path)
             nb = self.xml.get_widget("editor_notebook")
             nb.set_current_page(model.get_value(iter, 0))
-    
+
     def on_plugin_active_toggled(self, renderer, path):
         iter = self.plugin_model.get_iter(path)
         plugin = self.plugin_model.get_value(iter, 0)
@@ -176,7 +178,7 @@ class PreferencesDialog(GladeWidget):
         elif issubclass(plugin, GenericPlugin):
             self.plugin_model.set_value(iter, 1, not renderer.get_active())
             gobject.idle_add(self.app.plugins.set_active, plugin, not renderer.get_active())
-            
+
     def _plugin_download_and_activate(self, plugin):
         if not dialogs.yesno(_(u"Download plugin %r?") % plugin.xpath("//*/name")[0].text) == gtk.RESPONSE_YES:
             return
@@ -207,7 +209,7 @@ class PreferencesDialog(GladeWidget):
             log.error("URL: %s" % url)
             dlg.set_error(_(u"Failed to install plugin %r") % plugin.xpath("//*/name")[0].text)
         dlg.set_finished(True)
-            
+
     def on_plugin_added(self, manager, plugin):
         iter = self.plugin_model.get_iter_first()
         it = gtk.icon_theme_get_default()
@@ -231,7 +233,7 @@ class PreferencesDialog(GladeWidget):
                       3, ico,
                       4, True)
         self.refresh_plugins_repo()
-            
+
     def on_plugin_install(self, *args):
         dlg = gtk.FileChooserDialog(_(u"Install plugin"), None,
                                     gtk.FILE_CHOOSER_ACTION_OPEN,
@@ -249,7 +251,7 @@ class PreferencesDialog(GladeWidget):
             uri = None
         dlg.destroy()
         gobject.idle_add(self.app.plugins.install_plugin, uri)
-            
+
     def on_plugin_removed(self, manager, plugin):
         iter = self.plugin_model.get_iter_first()
         while iter:
@@ -262,7 +264,7 @@ class PreferencesDialog(GladeWidget):
                         return
                     citer = self.plugin_model.iter_next(citer)
             iter = self.plugin_model.iter_next(iter)
-            
+
     def on_plugin_prefs_show(self, *args):
         sel = self.plugin_list.get_selection()
         model, iter = sel.get_selected()
@@ -273,7 +275,7 @@ class PreferencesDialog(GladeWidget):
         if not obj.has_custom_options:
             return
         obj.run_custom_options_dialog(self.app)
-            
+
     def on_plugin_show_about(self, *args):
         sel = self.plugin_list.get_selection()
         model, iter = sel.get_selected()
@@ -291,7 +293,7 @@ class PreferencesDialog(GladeWidget):
         if obj.version: dlg.set_version(obj.version)
         dlg.run()
         dlg.destroy()
-            
+
     def on_plugin_selection_changed(self, selection, *args):
         model, iter = selection.get_selected()
         if not iter:
@@ -303,23 +305,25 @@ class PreferencesDialog(GladeWidget):
         else:
             self.xml.get_widget("plugin_about").set_sensitive(False)
             self.xml.get_widget("plugin_prefs").set_sensitive(False)
-            
+
     def on_plugin_sync_repo(self, *args):
         self.sync_repo_file()
-                
+
     def on_plugin_folder_show(self, *args):
         gnome.url_show(cf.USER_PLUGIN_URI)
-        
+
     def refresh(self):
         self.refresh_editor()
         self.refresh_plugins()
-        
+
     def refresh_editor(self):
         config = self.app.config
         gw = self.xml.get_widget
         gw("editor_open_separate").set_active(config.get("editor.open_in_window"))
         gw("editor_replace_variables").set_data("config_option", "editor.replace_variables")
         gw("editor_replace_variables").set_active(config.get("editor.replace_variables"))
+        gw("sqlparse_enabled").set_data("config_option", "sqlparse.enabled")
+        gw("sqlparse_enabled").set_active(config.get("sqlparse.enabled"))
         gw("editor_wrap_text").set_data("config_option", "editor.wrap_text")
         gw("editor_wrap_text").set_active(config.get("editor.wrap_text"))
         gw("editor_wrap_split").set_data("config_option", "editor.wrap_split")
@@ -350,7 +354,7 @@ class PreferencesDialog(GladeWidget):
         gw("editor_font_box").set_sensitive(not config.get("editor.default_font"))
         gw("editor_font").set_data("config_option", "editor.font")
         gw("editor_font").set_font_name(config.get("editor.font"))
-        
+
     def refresh_plugins(self):
         # Repo
         self.xml.get_widget("plugin_enable_repo").set_data("config_option", "plugins.repo_enabled")
@@ -362,7 +366,7 @@ class PreferencesDialog(GladeWidget):
         for key, value in self.app.plugins.plugin_types.items():
             iter = model.append(None)
             model.set(iter,
-                      0, key, 
+                      0, key,
                       1, False,
                       2, '<b>%s</b>' % value[0],
                       3, None,
@@ -385,7 +389,7 @@ class PreferencesDialog(GladeWidget):
                           3, ico,
                           4, not bool(plugin.INIT_ERROR))
         gobject.idle_add(self.refresh_plugins_repo)
-        
+
     def _plugin_iter_for_ep(self, ep_name):
         model = self.plugin_list.get_model()
         iter = model.get_iter_first()
@@ -394,7 +398,7 @@ class PreferencesDialog(GladeWidget):
                 return model, iter
             iter = model.iter_next(iter)
         return model, None
-            
+
     def refresh_plugins_repo(self):
         """Refresh repository plugins"""
         model = self.plugin_list.get_model()
@@ -446,7 +450,7 @@ class PreferencesDialog(GladeWidget):
                       2, lbl,
                       3, ico,
                       4, True)
-            
+
     def sync_repo_file(self, silent=False):
         """Synchronize plugin repo.xml"""
         def progress_cb(info, dlg):
@@ -456,6 +460,8 @@ class PreferencesDialog(GladeWidget):
                 fraction = 0
             dlg.set_progress(fraction)
             return True
+        # XXX currently disabled, should be replaced by a Capuchin backend
+        return
         url = os.path.join(self.app.config.get("plugins.repo_url"), "repo.xml")
         source = gnomevfs.URI(url)
         dest = gnomevfs.URI(cf.USER_PLUGIN_REPO_URI)
@@ -488,4 +494,4 @@ class PreferencesDialog(GladeWidget):
             retval = False
         if not silent:
             dlg.set_finished(True)
-        return retval
\ No newline at end of file
+        return retval
